/* freebob_bounce.h
 * Copyright (C) 2005 Pieter Palmers
 *
 * This file is part of FreeBoB
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 */

#ifndef FREEBOB_BOUNCE_H
#define FREEBOB_BOUNCE_H

#define FREEBOB_MAX_NAME_LEN 256

#define FREEBOB_BOUNCE_SERVER_VENDORNAME  "FreeBoB Server"
#define FREEBOB_BOUNCE_SERVER_MODELNAME   "freebob-server"

#define FREEBOB_BOUNCE_SERVER_GETXMLDESCRIPTION_CMD
#define AVC1394_SUBUNIT_TYPE_FREEBOB_BOUNCE_SERVER 	0x0D

#endif /* FREEBOB_BOUNCE_H */
